from bs4 import BeautifulSoup
from pyotp import TOTP
import json
from . import BaseClient


class Client(BaseClient):
    REQUIRES_AUTHENTICATION = False
    _REDIRECT_URL = "https://console.aws.amazon.com/console/home?state=hashArgs%23&isauthcode=true"

    def __init__(self, session):
        super().__init__(session)
        self._signin = self.session().client('signin_aws')

    def get_mfa_status(self, email):
        r = self.session()._post(
            "https://signin.aws.amazon.com/mfa",
            data={
                'email': email,
                '_redirect_url': self._REDIRECT_URL,
                'csrf': self._signin._csrf_token(),
                'sessionId': self._signin._session_id(),
            })

        if r.status_code != 200:
            raise Exception("failed get mfa status for {0}".format(email))

        return json.loads(r.text)



class BaseClient:
    REQUIRES_AUTHENTICATION = True

    def __init__(self, session):
        self._session = session

    def session(self):
        return self._session

from . import billing
from . import account
from . import federation
from . import iam
from . import mfa
from . import support
from . import resetpassword
from . import signin
from . import signin_amazon
from . import signin_aws



from bs4 import BeautifulSoup
from pyotp import TOTP
from urllib import parse
import json
from . import BaseClient


class Client(BaseClient):
    REQUIRES_AUTHENTICATION = False
    _REDIRECT_URL = "https://console.aws.amazon.com/console/home?state=hashArgs%23&isauthcode=true"

    def __init__(self, session):
        super().__init__(session)
        self.__csrf_token = None

    def _csrf_token(self):
        if self.__csrf_token == None:
            self._get_tokens()

        return self.__csrf_token

    def _get_tokens(self):
        r = self.session()._get(
            'https://signin.aws.amazon.com/resetpassword'
        )

        if r.status_code != 200:
            raise Exception("failed get tokens")

        soup = BeautifulSoup(r.text, 'html.parser')
        meta = {
            m['name']: m['content']
            for m in soup.find_all('meta') if 'name' in m.attrs
        }

        if not 'csrf_token' in meta:
            raise Exception("failed get csrf_token")
        self.__csrf_token = meta['csrf_token']

    def _action(self, action, data=None, api="signin"):
        """
        Execute an action on the signin API.

        Args:
            action: Action to execute.
            data: Arguments for the action.

        Returns:
            dict: Action response.
        """
        if not data:
            data = {}

        data['action'] = action
        # data['redirect_uri'] = self._REDIRECT_URL
        data['csrf'] = self._csrf_token()

        r = self.session()._post(
            "https://signin.aws.amazon.com/{0}".format(api),
            data=data,
        )

        if r.status_code != 200:
            print(r.text)
            raise Exception("failed action {0}".format(action))

        out = json.loads(r.text)
        if out['state'].lower() != 'success':
            if 'Message' in out['properties']:
                raise Exception("failed action {0}: {1}".format(action, out['properties']['Message']))
            else:
                raise Exception("failed action {0}".format(action))

        return out['properties']

    def reset_password(self, reset_token_url, password):
        """
        Performs a password reset.
        """
        query = parse.parse_qs(parse.urlparse(reset_token_url).query)
        return self._action('resetPasswordSubmitForm', {
            'token': query['token'][0],
            'key': query['key'][0],
            'newpassword': password,
            'confirmpassword': password,
        }, api='resetpassword')



from .. import BaseClient


class Client(BaseClient):
    REQUIRES_AUTHENTICATION = False

    def __init__(self, session):
        super().__init__(session)

        self._signin_aws = self.session().client('signin_aws')
        self._signin_amazon = self.session().client('signin_amazon')

    def signin(self, email, password, mfa_secret=None):
        # check account type
        account_type = self._signin_aws.get_account_type(email)

        if account_type == 'Decoupled':
            return self._signin_aws.signin(
                email,
                password,
                mfa_secret,
            )
        elif account_type == 'Coupled':
            return self._signin_amazon.signin(
                email,
                password,
                mfa_secret,
            )
        elif account_type == 'Unknown':
            raise Exception("account {0} not active".format(email))
        else:
            raise Exception("unsupported account type {0}".format(email))



from furl import furl
import json
import requests
from . import BaseClient


class Client(BaseClient):
    REQUIRES_AUTHENTICATION = False

    def __init__(self, session):
        super().__init__(session)

    def signin(self, boto3_session):
        """
        Signin using a boto3 session.

        This method uses the federation endpoint to obtain a signin token using
        the credentials in your boto3 session. The signin token is then used
        to signin into the AWS Management Console.

        Although possible, you are not encouraged to call this method directly,
        instead follow the following example.

        Example:
            .. code-block:: python

                import boto3
                import coto

                session = coto.Session(
                    boto3_session=boto3.Session()
                )

        Request Syntax:
            .. code-block:: python

                response = client.signin(
                    boto3_session=boto3.session.Session,
                )

        Args:
            boto3_session (boto3.session.Session): The boto3 session to use as
                provider for AWS credentials.

        Returns:
            bool: Signin succeeded.
        """
        r = self.session()._get(self.get_signin_url(boto3_session))
        if r.status_code != 200:
            raise Exception("failed session signin")

        self.session().authenticated = True
        return True

    def get_signin_url(self, boto3_session):
        """
        Signin using a boto3 session.

        This method uses the federation endpoint to obtain a signin token using
        the credentials in your boto3 session. The signin token is then used
        to signin into the AWS Management Console.

        Although possible, you are not encouraged to call this method directly,
        instead follow the following example.

        Example:
            .. code-block:: python

                import boto3
                import coto

                session = coto.Session(
                    boto3_session=boto3.Session()
                )

        Request Syntax:
            .. code-block:: python

                response = client.signin(
                    boto3_session=boto3.session.Session,
                )

        Args:
            boto3_session (boto3.session.Session): The boto3 session to use as
                provider for AWS credentials.

        Returns:
            bool: Signin succeeded.
        """
        url = furl('https://signin.aws.amazon.com/federation')

        url.args['Action'] = "login"
        url.args['Issuer'] = None
        url.args['Destination'] = "https://console.aws.amazon.com/"
        url.args['SigninToken'] = self.get_signin_token(boto3_session)

        return url.url

    def get_signin_token(self, boto3_session):
        """
        Obtain a signin token for a boto3 session.

        This method uses the federation endpoint to obtain a signin token using
        the credentials in your boto3 session.

        Request Syntax:
            .. code-block:: python

                response = client.get_signin_token(
                    boto3_session=boto3.session.Session,
                )

        Args:
            boto3_session (boto3.session.Session): The boto3 session to use as
                provider for AWS credentials.

        Returns:
            str: Signin token.
        """
        credentials = boto3_session.get_credentials()

        url = "https://signin.aws.amazon.com/federation"
        response = self.session()._get(
            url,
            params={
                "Action":
                "getSigninToken",
                "Session":
                json.dumps({
                    "sessionId": credentials.access_key,
                    "sessionKey": credentials.secret_key,
                    "sessionToken": credentials.token,
                })
            }
        )
        return json.loads(response.text)["SigninToken"]



from .captcha_guess import CaptchaGuess

class CaptchaRequiredException(Exception):
    def __init__(self, CES, CaptchaURL, captchaObfuscationToken, action):
        super().__init__(
            "Captcha required for action: {}".format(action)
        )

        self.CES = CES
        self.CaptchaURL = CaptchaURL
        self.captchaObfuscationToken = captchaObfuscationToken
        self.action = action

    def guess(self, guess):
        return CaptchaGuess(
            self.CES,
            self.captchaObfuscationToken,
            self.action,
            guess,
        )


from bs4 import BeautifulSoup
from pyotp import TOTP
from datetime import datetime, timedelta
import json
from . import BaseClient


class Client(BaseClient):
    """
    A low-level client representing IAM:

    .. code-block:: python

        import coto

        session = coto.Session()
        client = session.client('iam')

    These are the available methods:

    * General:
        * :py:meth:`get_account_info`

    * Access Keys:
        * :py:meth:`list_root_access_keys`
        * :py:meth:`create_root_access_key`
        * :py:meth:`update_root_access_key`
        * :py:meth:`delete_root_access_key`

    * MFA Devices:
        * :py:meth:`list_root_mfa_devices`
        * :py:meth:`create_virtual_mfa_device`
        * :py:meth:`enable_root_mfa_device`
        * :py:meth:`deactivate_root_mfa_device`
        * :py:meth:`delete_virtual_mfa_device`
    """

    def __init__(self, session):
        super().__init__(session)
        self.__xsrf_token = None

    def _url(self, api):
        return "https://console.aws.amazon.com/iam/service/{0}".format(api)

    def _xsrf_token(self):
        if self.__xsrf_token is None:
            self._get_xsrf_token()

        return self.__xsrf_token

    def _get_xsrf_token(self):
        r = self.session()._get(
            'https://console.aws.amazon.com/iam/home?&state=hashArgs%23')

        if r.status_code != 200:
            raise Exception("failed get token")

        r = self.session()._get('https://console.aws.amazon.com/iam/home?')

        if r.status_code != 200:
            raise Exception("failed get token")

        soup = BeautifulSoup(r.text, 'html.parser')
        for m in soup.find_all('meta'):
            if 'id' in m.attrs and m['id'] == "xsrf-token":
                self.__xsrf_token = m['data-token']
                return

        raise Exception('unable to obtain IAM xsrf_token')

    def _get(self, api):
        r = self.session()._get(
            self._url(api), headers={'X-CSRF-Token': self._xsrf_token()})

        if 'X-CSRF-Token' in r.headers:
            self.__xsrf_token = r.headers['X-CSRF-Token']

        if r.status_code != 200:
            print(r.text)
            raise Exception("failed get {0}".format(api))

        return json.loads(r.text)

    def _post(self, api, data=None):
        r = self.session()._post(
            self._url(api),
            headers={
                'X-CSRF-Token': self._xsrf_token(),
                'Content-Type': 'application/json',
            },
            data=json.dumps(data) if data is not None else None,
        )

        if 'X-CSRF-Token' in r.headers:
            self.__xsrf_token = r.headers['X-CSRF-Token']

        if r.status_code != 200:
            print(r.text)
            raise Exception("failed post {0}".format(api))

        return json.loads(r.text)

    def _http(self, method, api, data=None):
        r = self.session()._post(
            self._url(api),
            headers={
                'X-CSRF-Token': self._xsrf_token(),
                'x-http-method-override': method.upper(),
            },
            data=json.dumps(data) if data is not None else None,
        )

        if 'X-CSRF-Token' in r.headers:
            self.__xsrf_token = r.headers['X-CSRF-Token']

        if r.status_code != 200:
            print(r.text)
            raise Exception("failed delete {0}".format(api))

        return json.loads(r.text)

    # iam api

    def get_account_info(self):
        """
        Retrieves a summary of account information.

        Request Syntax:
            .. code-block:: python

                response = client.get_account_info()

        Returns:
            dict: Response Syntax

            .. code-block:: python

                {
                    'aliases': [],
                    'checklistSummary': {
                        'checklistItems': [
                            {
                                'complete': bool,
                                'fetchSucceeded': bool,
                                'identifier': str,
                            },
                        ],
                        'error': bool,
                        'errorCount': int,
                        'totalCompletedCount': int,
                        'totalCount': int
                    },
                    'errorMap': {},
                    'errors': [],
                    'invalidPolicyExist': bool,
                    'summaryMap': {
                        'AccessKeysPerUserQuota': int,
                        'AccountAccessKeysPresent': int,
                        'AccountMFAEnabled': int,
                        'AccountSigningCertificatesPresent': int,
                        'AssumeRolePolicySizeQuota': int,
                        'AttachedPoliciesPerGroupQuota': int,
                        'AttachedPoliciesPerRoleQuota': int,
                        'AttachedPoliciesPerUserQuota': int,
                        'GroupPolicySizeQuota': int,
                        'Groups': int,
                        'GroupsPerUserQuota': int,
                        'GroupsQuota': int,
                        'InstanceProfiles': int,
                        'InstanceProfilesQuota': int,
                        'MFADevices': int,
                        'MFADevicesInUse': int,
                        'Policies': int,
                        'PoliciesQuota': int,
                        'PolicySizeQuota': int,
                        'PolicyVersionsInUse': int,
                        'PolicyVersionsInUseQuota': int,
                        'Providers': int,
                        'RolePolicySizeQuota': int,
                        'Roles': int,
                        'RolesQuota': int,
                        'ServerCertificates': int,
                        'ServerCertificatesQuota': int,
                        'SigningCertificatesPerUserQuota': int,
                        'UserPolicySizeQuota': int,
                        'Users': int,
                        'UsersQuota': int,
                        'VersionsPerPolicyQuota': int,
                    }
                }
        """
        return self._get('account')

    def list_root_mfa_devices(self):
        """
        Lists enabled root MFA devices.

        Request Syntax:
            .. code-block:: python

                response = client.list_root_mfa_devices()

        Returns:
            dict: Response Syntax

            .. code-block:: python

                {
                    'serialNumber': [
                        str,
                    ],
                    'nextItem': str,
                    'truncated': bool
                }
        """
        r = self._get('root/mfa/')
        return r

    def create_virtual_mfa_device(
            self, VirtualMFADeviceName='root-account-mfa-device', Path='/'):
        """
        Creates a new virtual MFA device for the AWS account. After creating the
        virtual MFA, use :py:meth:`enable_mfa_device` to attach the MFA device to the
        account root user.

        Request Syntax:
            .. code-block:: python

                response = client.create_virtual_mfa_device(
                    VirtualMFADeviceName=str,
                    Path=str
                )

        Args:
            VirtualMFADeviceName (str): The name of the virtual MFA device. Use with path to uniquely identify a virtual MFA device.
                This parameter is optional. If it is not included, it defaults to ``root-account-mfa-device``.
            Path (str): The path for the virtual MFA device. For more information about paths, see IAM Identifiers in the IAM User Guide.
                This parameter is optional. If it is not included, it defaults to a slash (/).

        Returns:
            dict: Response Syntax

            .. code-block:: python

                {
                    "serialNumber": str,
                    "qrCodePNG": str,
                    "base32StringSeed": str
                }

            **serialNumber** (*str*) -- The serial number associated with VirtualMFADevice.

            **qrCodePNG** (*str*) -- A QR code PNG image that encodes
            ``otpauth://totp/$virtualMFADeviceName@$AccountName?secret=$Base32String``
            where ``$virtualMFADeviceName`` is one of the create call arguments,
            ``$AccountName`` is the user name if set (otherwise, the account
            ID), and ``$Base32String`` is the seed in Base32 format. The
            ``$Base32String`` value is Base64-encoded.

            **base32StringSeed** (*str*) -- The Base32 seed defined as specified
            in RFC3548 . The Base32StringSeed is Base64-encoded.
        """
        r = self._post('mfa/createVirtualMfa', {
            'path': Path,
            'deviceName': VirtualMFADeviceName
        })
        return r

    def enable_root_mfa_device(self,
                               SerialNumber,
                               Base32StringSeed=None,
                               AuthenticationCode1=None,
                               AuthenticationCode2=None):
        """
        Enables the specified MFA device and associates it with the account root
        user. When enabled, the MFA device is required for every subsequent
        login by the account root user.

        Specify either the ``Base32StringSeed``, or both ``AuthenticationCode1``
        and ``AuthenticationCode2``.

        Request Syntax:
            .. code-block:: python

                response = client.enable_root_mfa_device(
                    SerialNumber=str,
                    Base32StringSeed=str,
                )

            or

            .. code-block:: python

                response = client.enable_root_mfa_device(
                    SerialNumber=str,
                    AuthenticationCode1=str,
                    AuthenticationCode2=str,
                )

        Args:
            SerialNumber (str): The serial number that uniquely identifies the
                MFA device. For virtual MFA devices, the serial number is the
                device ARN.
            Base32StringSeed (str): The Base32 seed defined as specified in
                RFC3548. The Base32StringSeed is Base64-encoded. If set, the
                the current values for the ``AuthenticationCode1`` and
                ``AuthenticationCode2`` arguments will be calculated.
            AuthenticationCode1 (str): An authentication code emitted by the
                device. The format for this parameter is a string of 6 digits.
                If ``Base32StringSeed`` is set, it wil override this argument.
            AuthenticationCode2 (str): An authentication code emitted by the
                device. The format for this parameter is a string of 6 digits.
                If ``Base32StringSeed`` is set, it wil override this argument.

        Returns:
            bool: success
        """
        if Base32StringSeed:
            current = datetime.now()
            previous = current - timedelta(seconds=30)
            totp = TOTP(Base32StringSeed)
            AuthenticationCode1 = totp.at(previous)
            AuthenticationCode2 = totp.at(current)

        r = self._post(
            'root/mfa/associate', {
                'serial': SerialNumber,
                'codes': [AuthenticationCode1, AuthenticationCode2]
            })
        return r['success']

    def deactivate_root_mfa_device(self, SerialNumber):
        """
        Deactivates the specified MFA device and removes it from association
        with the account root user.

        Request Syntax:
            .. code-block:: python

                response = client.deactivate_root_mfa_device(
                    SerialNumber=str
                )

        Args:
            SerialNumber (str): The serial number that uniquely identifies the
                MFA device. For virtual MFA devices, the serial number is the
                device ARN.
        """
        r = self._post('root/mfa/disassociate', {'serial': SerialNumber})
        return r

    def delete_virtual_mfa_device(self, SerialNumber):
        """
        Deletes the specified virtual MFA device.

        Request Syntax:
            .. code-block:: python

                response = client.delete_virtual_mfa_device(
                    SerialNumber=str
                )

        Args:
            SerialNumber (str): The serial number that uniquely identifies the
                MFA device. For virtual MFA devices, the serial number is the
                device ARN.
        """
        r = self._http('delete', 'proxy/DeleteVirtualMFADevice',
                       {'serialNumber': SerialNumber})
        return r

    def list_root_access_keys(self, Deleted=False):
        """
        List the access key pairs associated with the account root user.

        Request Syntax:
            .. code-block:: python

                response = client.list_root_access_keys(
                    Deleted=bool,
                )

        Args:
            Deleted (bool): List the deleted access key pairs
        Returns:
            dict: Response Syntax

            .. code-block:: python

                [
                    {
                        'createDate': int,
                        'deleteDate': int,
                        'id': int,
                        'lastUsedDetails': {
                            'lastDateUsed': int,
                            'region': str,
                            'serviceName': str,
                        },
                        'status': 'Active' | 'Inactive' | 'Deleted',
                    },
                ]
        """
        if Deleted:
            r = self._get('root/keys/?deleted=1')
        else:
            r = self._get('root/keys')
        return r

    def create_root_access_key(self):
        """
        Creates a new AWS secret access key and corresponding AWS access key ID
        for the account root user. The default status for new keys is Active.

        Request Syntax:
            .. code-block:: python

                response = client.create_root_access_key()

        Returns:
            dict: Response Syntax

            .. code-block:: python

                {
                    "id": str,
                    "status": "Active",
                    "secret": str,
                    "createDate": int,
                    "deleteDate": int
                }
        """
        r = self._post('root/keys')
        return r

    def update_root_access_key(self, AccessKeyId, Status='Inactive'):
        """
        Changes the status of the specified access key from Active to Inactive,
        or vice versa. This action can be used to disable a account root user's
        key as part of a key rotation work flow.

        Request Syntax:
            .. code-block:: python

                response = client.activate_root_access_key(
                    AccessKeyId=str,
                    Status='Active' | 'Inactive',
                )

        Args:
            AccessKeyId (str): The access key ID for the access key ID and
                secret access key you want to activate.
            Status (str): The status you want to assign to the secret access
                key. Active means the key can be used for API calls to AWS,
                while Inactive means the key cannot be used.

        Returns:
            bool: success
        """
        if Status.lower() == 'active':
            r = self._http('activate', "root/keys/{0}".format(AccessKeyId))
        else:
            r = self._http('deactivate', "root/keys/{0}".format(AccessKeyId))
        return r['success']

    def delete_root_access_key(self, AccessKeyId):
        """
        Deletes the access key pair associated with the account root user.

        Request Syntax:
            .. code-block:: python

                response = client.delete_root_access_key(
                    AccessKeyId=str,
                )

        Args:
            AccessKeyId (str): The access key ID for the access key ID and
                secret access key you want to delete.

        Returns:
            bool: success
        """
        r = self._http('delete', "root/keys/{0}".format(AccessKeyId))
        return r['success']



from bs4 import BeautifulSoup
from pyotp import TOTP
from urllib import parse
import json
from .. import BaseClient
import time
from furl import furl


# import os
# import webbrowser

# def view_html(html):
#     path = os.path.abspath('temp.html')
#     url = 'file://' + path
#     with open(path, 'w') as f:
#         f.write(html)
#     webbrowser.open(url)

def ap_url(email):
    url = furl("https://www.amazon.com/ap/signin")

    url.args["openid.assoc_handle"] = "aws"
    url.args["openid.return_to"] = "https://signin.aws.amazon.com/oauth?coupled_root=true&response_type=code&redirect_uri=https%3A%2F%2Fconsole.aws.amazon.com%2Fconsole%2Fhome%3Fstate%3DhashArgs%2523%26isauthcode%3Dtrue&client_id=arn%3Aaws%3Aiam%3A%3A015428540659%3Auser%2Fhomepage"
    url.args["openid.mode"] = "checkid_setup"
    url.args["openid.ns"] = "http://specs.openid.net/auth/2.0"
    url.args["openid.identity"] = "http://specs.openid.net/auth/2.0/identifier_select"
    url.args["openid.claimed_id"] = "http://specs.openid.net/auth/2.0/identifier_select"
    url.args["action"] = ""
    url.args["disableCorpSignUp"] = ""
    url.args["clientContext"] = ""
    url.args["marketPlaceId"] = ""
    url.args["poolName"] = ""
    url.args["authCookies"] = ""
    url.args["pageId"] = "aws.login"
    url.args["siteState"] = "registered,EN_US"
    url.args["accountStatusPolicy"] = "P1"
    url.args["sso"] = ""
    url.args["openid.pape.preferred_auth_policies"] = "MultifactorPhysical"
    url.args["openid.pape.max_auth_age"] = "120"
    url.args["openid.ns.pape"] = "http://specs.openid.net/extensions/pape/1.0"
    url.args["server"] = "/ap/signin?ie=UTF8"
    url.args["accountPoolAlias"] = ""
    url.args["forceMobileApp"] = "1"
    url.args["language"] = "EN_US"
    url.args["forceMobileLayout"] = "0"
    url.args["awsEmail"] = email

    return url.url


class Client(BaseClient):
    REQUIRES_AUTHENTICATION = False
    _REDIRECT_URL = "https://console.aws.amazon.com/console/home?state=hashArgs%23&isauthcode=true"

    def __init__(self, session):
        super().__init__(session)

    def find_and_submit_form(self, soup, email, password, mfa_secret=None):
        error = soup.find(id="message_error")
        if error:
            message = error.get_text()
            # Enter the characters as they are given in the challenge.
            raise Exception(message)

        form = soup.find(id="ap_signin_form")

        if not form:
            mfa_form = soup.find(id="auth-mfa-form")
            if mfa_form:
                raise Exception("accounts with Amazon MFA not supported")

        data = {'metadata1': self.session()._metadata1_generator.generate()}

        for field in form.find_all('input'):
            name = field.get('name')
            if not name:
                continue
            value = field.get('value')
            data[name] = value
                
        if "guess" in data:
            if not self.session()._captcha_solver:
                raise Exception("captcha solver required")

            captcha = soup.find(id="ap_captcha_img")
            img = captcha.find("img")
            src = img.get("src")
            guess_uuid = self.session()._captcha_solver.solve(url=src)

            while True:
                guess = self.session()._captcha_solver.result(guess_uuid)

                if guess is None:
                    time.sleep(5)
                else:
                    data["guess"] = guess
                    break

        if "tokenCode" in data and mfa_secret:
            data['tokenCode'] = TOTP(mfa_secret).now()

        overrides = {
            "password": password,
            "email": email,
        }

        for k, v in data.items():
            if v is None:
                _v = overrides.get(k)
                if _v:
                    data[k] = _v

        return self.session()._post(
            form.get("action"),
            data=data
        )

    def signin(self, email, password, mfa_secret=None):
        """
        Signin into the AWS Management Console using account root user.

        Request Syntax:

            .. code-block:: python

                response = client.get_account_type(
                    email=str,
                    password=str,
                    mfa_secret=str,
                )

        Args:
            email: Account email address.
            password: Account password.
            mfa_secret: Account mfa secret. The Base32 seed defined as specified
                in RFC3548. The Base32StringSeed is Base64-encoded.

        Returns:
            bool: Signin successful
        """

        # first post password
        response = self.session()._get(ap_url(email))
        soup = BeautifulSoup(response.text, 'html.parser')
        response = self.find_and_submit_form(soup, email, password, mfa_secret)
        # view_html(response.text)

        counter = 0

        while counter < 10 and response.url != "https://console.aws.amazon.com/console/home":
            counter += 1
            soup = BeautifulSoup(response.text, 'html.parser')
            response = self.find_and_submit_form(soup, email, password, mfa_secret)

        if response.url == "https://console.aws.amazon.com/console/home":
            self.session().authenticated = True
            self.session().root = True
            return True
        else:
            return False
